#include <stdio.h>
#include <math.h>
#include "bldc.h"

#define MAXC 4000
#define STEP (MAXC/200)

#define LEAD MAXC/8
#define ULEAD (MAXC*75/360)
#define VLEAD (MAXC*135/360)
#define WLEAD (MAXC*195/360)

#define MAXWAIT 1000
#define MINWAIT 100

#define NTICKS 40
static int ticks[NTICKS];
static int tidx = 0;

main()
{
  int i;
  int halls, old_halls = -1;
  int u = 0;
  int v = 0;
  int w = 0;
  int power = 0;
  int pos, old_pos = -999;
  int running = 0;
  int update_status = 1;
  int then = 0;
  int lead = -45;
  int dloop = 0;
  int odloop = 0;
  int target = 0, old_target = 0;
  int time_last_pos_change = 0;
  int idle_threshold = 4;

  bldc_init ();
  encoder_init ();
  cpwm_init ();
  thermistor_init ();

  tty_puts("\033[1;1H\033[2J\n\nHello from Step 2!\n");

  /* Shut off the motor.  */
  PORTB.DR.BYTE = UL | VL | WL;

  simple_commutate (0);

  printf("Looping...\n");

  while (1)
    {
	  int move, lead;
	  double torque;

      if (tty_ready ())
	{
	  int c = tty_getc ();
	  switch (c)
	    {
	    case 'r':
	      bldc_run ();
	      old_pos = -999;
	      running = 1;
	      break;

	    case 's':
	    case ' ':
	      bldc_stop ();
	      old_pos = -999;
	      running = 0;
	      break;

	    case '0' ... '9':
	      if (c == '0')
		power = 32700;
	      else
		power = (c - '0') * 32767/10;
	      old_pos = -999;
	      break;

	    case '+':
	      target -= STEP * 9;
	    case '=':
	      target -= STEP;
	      if (target < 0)
		target += MAXC;
	      tidx = 0;
	      idle_threshold = 3;
	      break;

	    case '_':
	      target += STEP * 9;
	    case '-':
	      target += STEP;
	      if (target >= MAXC)
		target -= MAXC;
	      tidx = 0;
	      idle_threshold = 3;
	      break;

	    case 't':
	      printf("\033[5;1H");
	      then = ticks[0];
	      for (i=1; i<tidx; i++)
		printf("%3d %6d\n", i, (short)ticks[i] - (short)then);
	      printf("\033[J");
	      fflush (stdout);
	      break;

	    }
	}

      pos = encoder_val ();

      if (pos != old_pos || target != old_target)
	{

	  old_pos = pos;
	  old_target = target;
	  update_status = 1;

	  if (tidx < NTICKS)
	    ticks[tidx++] = msec;
	  time_last_pos_change = msec;

	  move = pos - target;
	  if (move > (MAXC/2))
	    move -= MAXC;
	  else if (move < -(MAXC/2))
	    move += (MAXC);

	  if (move > idle_threshold)
	    {
	      torque = move/(MAXC/8.0) + 0.05;
	      if (torque > 1) torque = 1;
	      lead = -(MAXC/8);
	    }
	  else if (move < -idle_threshold)
	    {
	      torque = move/(MAXC/8.0) - 0.05;
	      if (torque < -1) torque = -1;
	      lead = -(MAXC/8);
	    }
	  else
	    {
	      torque = 0;
	      idle_threshold = 10;
	    }

	  cpwm_torque (power * torque);
	}
      else if ((msec - time_last_pos_change) > 100)
	{
	  if (abs (move) < idle_threshold)
	    {
	      cpwm_torque (0);
	      idle_threshold = 200;
	    }
	  else
	    cpwm_torque (power * ((torque > 0) ? 0.1 : -0.1));
	  time_last_pos_change = msec;
	  update_status = 1;
	}

      if ((update_status || (msec - then) > MAXWAIT || pwm_status_new)
	  && tty_idle () && (msec - then) > MINWAIT)
	{
	  then = msec;
	  update_status = 0;
	  printf("\033[1;1H  pos = %5d vs %5d, \033[%dmpower = %5d\033[0m  ",
		 pos, target, running ? 32 : 31, power);

	  temperature();

	  printf("\033[K");

	  if (pwm_status_new)
	    {
	      fputs (pwm_status, stdout);
	      pwm_status_new = 0;
	    }

	  fflush (stdout);
	}
    }

  exit (0);
}
