#include "bldc.h"

static volatile int val_high_word = 0;

static void __attribute__((interrupt))
overflow_handler ()
{
  val_high_word ++;
  (void)MTU2.TSR.BYTE;
  MTU2.TSR.BIT.TCFV = 0;
}

static void __attribute__((interrupt))
underflow_handler ()
{
  val_high_word --;
  (void)MTU2.TSR.BYTE;
  MTU2.TSR.BIT.TCFU = 0;
}

void
step_direction_init ()
{
  MTU2.TCNT = 0;
  MTU2.TMDR.BIT.MD = 5; /* phase counting mode 2 */
  MTU.TSTRA.BIT.CST2 = 1; /* start MTU2 */

  _vectors[VECT(MTU2,TCIV2)] = overflow_handler;
  _vectors[VECT(MTU2,TCIU2)] = underflow_handler;
  MTU2.TIER.BIT.TCIEU = 1;
  MTU2.TIER.BIT.TCIEV = 1;
  IEN(MTU2,TCIV2) = 1;
  IPR(MTU2,TCIV2) = 5;
  IEN(MTU2,TCIU2) = 1;
  IPR(MTU2,TCIU2) = 5;
}

long long
step_direction_lval ()
{
  unsigned short count;
  int r;

  do {
    r = val_high_word;
    count = MTU2.TCNT;
  } while (r != val_high_word);

  return ((long long)r << 16) | count;
}

void
step_direction_reset ()
{
  MTU2.TCNT = 0;
  val_high_word = 0;
}

/* Remember, called 20k times per second.  This is 0.1 second.  */
#define DEBOUNCE_TIME 2000
static int debounce1 = DEBOUNCE_TIME;
static int debounce2 = DEBOUNCE_TIME;

void
limit_pwm_update ()
{
  if (SLIMIT1_HIT())
    debounce1 = 0;
  else if (debounce1 < DEBOUNCE_TIME)
    debounce1++;

  if (SLIMIT2_HIT())
    debounce2 = 0;
  else if (debounce2 < DEBOUNCE_TIME)
    debounce2++;

  if (debounce1 < DEBOUNCE_TIME)
    SD_LIMIT1 = (config_limits.limit1_active_low ? 0 : 1);
  else
    SD_LIMIT1 = (config_limits.limit1_active_low ? 1 : 0);

  if (debounce2 < DEBOUNCE_TIME)
    SD_LIMIT2 = (config_limits.limit2_active_low ? 0 : 1);
  else
    SD_LIMIT2 = (config_limits.limit2_active_low ? 1 : 0);
}
